/*
 * Copyright (c) 2017 Linagora.
 *
 * This file is part of Hublot
 * (see https://ci.linagora.com/linagora/lgs/labs/hublot).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

// Start a netcat socket mirror
// ncat --broker --listen -p PORT_NUMBER : dispatch anydata to all
// nc -l -p PORT_NUMBER
const {
  EventEmitter
} = require('events');
const net = require('net');
const debug = require('debug')('lib:asd');

class Asd extends EventEmitter {
  constructor() {
    super();
    return this.init();
  }

  async init() {
    return new Promise(async (resolve, reject) => {
      try {
        this.streams = [];
        await this.controlConnect();
        resolve(this);
      } catch (e) {
        // Eventual catch of explicit rejections in awaited function
        reject(e);
      }
    });
  }

  controlConnect() {
    return new Promise((resolve, reject) => {
      let recoTimer;
      // Tries to reconnect in case of error
      function reco() {
        recoTimer = setTimeout(() => {
          this.init();
        }, 2000);
      }

      try {
        const controlTcpClient = new net.Socket();
        controlTcpClient.once('error', err => {
          debug(err);
          // Calls the timer for reconnection
          reco.bind(this)();
        });
        controlTcpClient.connect(process.env.ASD_CONTROL_PORT, process.env.ASD_SERVER_IP, socket => {
          clearTimeout(recoTimer);
          debug(`ASD Control connexion established to ${process.env.ASD_SERVER_IP} : ${process.env.ASD_CONTROL_PORT}`);
          controlTcpClient.on('close', () => {
            debug('ASD Control connection closed');
          });
          controlTcpClient.on('data', data => {
            debug('Received and emited: ' + data.toString());
            this.emit('resume_data', data);
          });
          resolve(true);
        });
      } catch (e) {
        reco.bind(this)();
      }
    });
  }

  streamConnect() {
    return new Promise((resolve, reject) => {
      const streamClient = new net.Socket();
      let streamId;
      streamClient.connect(process.env.ASD_STREAM_PORT, process.env.ASD_SERVER_IP, socket => {
        streamId = Symbol();
        this.streams[streamId] = streamClient;
        debug('New stream connected');
        streamClient.on('close', err => {
          delete this.streams[streamId];
          debug('STREAM DISCONNECTED !');
          this.emit('stream_disco', streamId);
        });
        resolve(streamId);
      });

      streamClient.on('error', err => {
        if (this.streams[streamId]) {
          delete this.streams[streamId];
        }
        debug('STREAM CRASHED !');
        this.emit('stream_crash', streamId);
      });
    });
  }

  streamPush(streamId, data) {
    this.streams[streamId].write(data);
  }

  streamDisconnect(streamId) {
    this.streams[streamId].destroy();
    delete this.streams[streamId];
    this.emit('stream_closed', streamId);
  }
}

module.exports = new Asd();
