/*
 * Copyright (c) 2017 Linagora.
 *
 * This file is part of Hublot
 * (see https://ci.linagora.com/linagora/lgs/labs/hublot).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

const http = require('http');
const SockJS = require('sockjs-client');
const stomp = require('webstomp-client');
const WebSocket = require('ws');
const XMLHttpRequest = require('xmlhttprequest').XMLHttpRequest;

module.exports = config => {
  function createStompClient() {
    const stompCapsule = {
      get: null
    };

    function tryConnect() {
      console.log('proxy: connecting to STOMP...');

      const sock = new SockJS('http://' + config.services.reco.host + ':' + config.services.reco.port + '/chat');
      stompCapsule.get = stomp.over(sock, {
        debug: false
      });

      stompCapsule.get.connect(
        {},
        () => console.log('proxy: STOMP connected'),
        err => {
          console.error(
            'proxy: STOMP failed to connect to %s:%s (trying again in %d ms)',
            config.services.reco.host, config.services.reco.port, config.services.reco.reconnectInterval);
          console.error(err);
          setTimeout(tryConnect, config.services.reco.reconnectInterval);
        });
    }
    tryConnect();

    return stompCapsule;
  }

  function recoChatWS(connectionToClient, stompClient) {
    connectionToClient.on('message', message => {
      try {
        stompClient.get.send('/app/chat', message);
      } catch (err) {
        console.error('proxy: chatWS', err);
      }
    });

    connectionToClient.on('error', error => {
      console.error('proxy: WS chat error', error);
    });
  }

  function recoStartStopWS(connectionToClient) {
    connectionToClient.on('message', message => {
      const xmlHttp = new XMLHttpRequest();
      const url = 'http://' + config.services.reco.host + ':' + config.services.reco.port + message;
      xmlHttp.open('GET', url, true);
      xmlHttp.setRequestHeader('Content-type', 'application/json');
      xmlHttp.send(null);

      connectionToClient.close();
    });

    connectionToClient.on('error', error => {
      console.error('proxy: WS reco error', error);
    });
  }

  function recoRecoWS(connectionToClient) {
    connectionToClient.on('message', message => {
      const xmlHttp = new XMLHttpRequest();

      xmlHttp.onreadystatechange = () => {
        if (xmlHttp.readyState === 4) {
          if (xmlHttp.status === 200) {
            try {
              connectionToClient.send(xmlHttp.responseText);
            } catch (err) {
              console.error('proxy: recoRecoWS error', err);
            }
          } else {
            console.error('Online reco: error trying to reach http://%s:%s/resources',
              config.services.reco.host, config.services.reco.port);
          }
          connectionToClient.close();
        }
      };

      const url = 'http://' + config.services.reco.host + ':' + config.services.reco.port + message;
      xmlHttp.open('GET', url, true);
      xmlHttp.setRequestHeader('Content-type', 'application/json');
      xmlHttp.send(null);
    });

    connectionToClient.on('error', error => {
      console.error('proxy: WS reco error', error);
    });
  }

  function kaldiWS(userId, roomId, socket) {
    const connectionToKaldi = new WebSocket(config.services.kaldi);
    connectionToKaldi.on('open', () => {
      connectionToKaldi.on('message', message => {
        try {
          const msgJson = JSON.parse(message);
          msgJson.userId = userId;
          msgJson.roomId = roomId;

          console.log('room transcription ' + roomId + ' ==== ', msgJson);
          socket.emit('resume_data_' + roomId, msgJson);
        } catch (err) {
          // Sometimes, the client will close the connection unexpectedly
          // (e.g. when stopping). This is expected
        }
      });
    });
    return connectionToKaldi;
  }

  function audioSocket(resume) {
    const audioWebSocketServer = http.createServer();

    const io = require('socket.io')(audioWebSocketServer, {
      transports: ['websocket']
    });

    io.sockets.on('connection', async socket => {
      let asd;
      let userAudioEvents = [];

      if (config.asd.isUse) {
        asd = await require('./asd');
        asd.on('resume_data', data => {
          // Envoyer la data vers Hublin
          socket.emit('resume_data', data);
        });
      }

      socket.on('error', err => {
        console.log('Error trace : ', err);
        for (const userEvent in userAudioEvents) {
          socket.removeListener(userEvent);
        }
        userAudioEvents = [];
      });

      socket.on('new_user', async data => {
        const userId = data.userId;
        const roomId = data.roomId;
        resume.createFolder('_' + roomId);

        const streamData = resume.generateStream(roomId);
        let transcriptor;

        if (config.asd.isUse) {
          transcriptor = await asd.streamConnect();
        } else {
          transcriptor = kaldiWS(userId, roomId, socket);
        }

        userAudioEvents.push('audio_from_' + userId);

        // Register custom event for user
        socket.on('audio_from_' + userId, data => {
          if (config.asd.isUse) {
            asd.streamPush(transcriptor, data); // Asd
          } else {
            if (transcriptor.readyState === 2 || transcriptor.readyState === 3) {
              transcriptor = kaldiWS(userId, roomId, socket);
            }
            if (transcriptor.readyState === 1) {
              transcriptor.send(data, {binary: true});
            } // Kaldi
          }

          streamData.wstream.write(data);
        });

        socket.on('disconnect_from_' + userId, reason => {
          console.log('Disconnect reason is : ', reason);
          resume.endStream(streamData);
          delete userAudioEvents['audio_from_' + userId];

          if (config.asd.isUse) {
            asd.streamDisconnect(transcriptor);// Asd
          } else {
            transcriptor.close(); // Kaldis
          }
        });
      });
    });

    audioWebSocketServer.listen(config.port_ws_audio);
  }

  return {
    create: resume => {
      const stompClient = createStompClient();
      audioSocket(resume);

      const proxyEndpointServer = http.createServer();

      const wssServer = new WebSocket.Server({
        server: proxyEndpointServer,
        autoAcceptConnections: true
      });

      wssServer.on('connection', (connectionToClient, req) => {
        if (req.url === '/chat') {
          recoChatWS(connectionToClient, stompClient);
          return;
        }
        if (req.url === '/startstop') {
          recoStartStopWS(connectionToClient);
          return;
        }
        if (req.url === '/reco') {
          recoRecoWS(connectionToClient);
          return;
        }
        console.error('proxy: unknown path for ws connection', req.url);
      });
      proxyEndpointServer.listen(config.port_server_proxy);
    }
  };
};
