/*
 * Copyright (c) 2017 Linagora.
 *
 * This file is part of Hublot
 * (see https://ci.linagora.com/linagora/lgs/labs/hublot).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
const fs = require('fs');
const util = require('util');
const uuidv1 = require('uuid/v1');
const request = require('request');

const ffmpegInstaller = require('@ffmpeg-installer/ffmpeg');
const ffprobeInstaller = require('@ffprobe-installer/ffprobe');

const ffmpeg = require('fluent-ffmpeg');

ffmpeg.setFfmpegPath(ffmpegInstaller.path);
ffmpeg.setFfprobePath(ffprobeInstaller.path);

const successCode = 200;

module.exports = ffmpeg;

const self = module.exports = (config, resumeConfig) => {
  let timestamp;
  const userConf = {};

  function parserTranscription(transcription) {
    const lines = transcription.split('\n');
    const json = {
      entries: []
    };

    for (let i = 0; i < lines.length - 1; i++) {
      const data = lines[i].replace(/\\/gi, '').split(' ');

      const speaker = data[1];
      const from = data[2];
      const until = data[3];
      let text = '';
      for (let j = 4; j < data.length; j++) {
        text = text + data[j] + ' ';
      }

      const jsonData = {
        from,
        until,
        speaker,
        text
      };
      json.entries.push(jsonData);
    }
    return json;
  }

  function offlineTranscription(filename, enhancementlCallback) {
    const url = 'http://' + config.host + ':' + config.port + '/' + config.api.upload;

    const options = {
      url,
      formData: {
        wavFile: {
          value: fs.ReadStream(config.folder.audioMerged + '/' + filename),
          options: {
            filename,
            contentType: 'audio/x-wav'
          }
        },
        model: config.model
      },
      encoding: null
    };

    console.log('POST OFFLINE TRANSCRIPTION');
    request.post(options, enhancementlCallback);
  }

  function resume(json, roomId) {
    const url = 'http://' + resumeConfig.hostName + '/' + resumeConfig.api + '?id=test' + resumeConfig.query;

    const options = {
      uri: url,
      method: 'POST',
      json
    };

    console.log('POST RESUME');
    request(options, (error, response, body) => {
      console.log(body);
      if (!error && response.statusCode === successCode) {
        fs.writeFile(resumeConfig.resumeFolder + '/resume_' + roomId + '.txt', util.inspect(body), 'utf-8', error => {
          if (error !== null) {
            console.log(error);
          }
        });
      }
      if (body === undefined) {
        console.log('Summary was empty');
      }
    });
  }

  function resumeGenerator(roomId) {
    let nbFile = 0;
    let memoryFile;
    const cmd = ffmpeg();
    fs.readdirSync(config.folder.audioMerged + '_' + roomId).forEach(file => {
      cmd.input(config.folder.audioMerged + '_' + roomId + '/' + file);
      nbFile++;
      memoryFile = file;
    });
    if (nbFile !== 1) {
      memoryFile = '/merged.wav';
    }

    cmd.complexFilter('amix=inputs=' + nbFile + ':duration=first:dropout_transition=3')
      .save(config.folder.audioMerged + '_' + roomId + '/merged.wav')
      .on('end', () => {
        // OfflineTranscription(memoryFile, (err, response, body) => {
        // const textChunk = body.toString('utf8');
        let textChunk;
        const jsonTranscript = parserTranscription(textChunk);
        resume(jsonTranscript, roomId);
        // });
      });
  }

  return {
    createFolder: room => {
      if (!fs.existsSync(config.folder.audioRaw + room)) {
        fs.mkdirSync(config.folder.audioRaw + room);
      }
      if (!fs.existsSync(config.folder.audioWav + room)) {
        fs.mkdirSync(config.folder.audioWav + room);
      }
      if (!fs.existsSync(config.folder.audioMerged + room)) {
        fs.mkdirSync(config.folder.audioMerged + room);
      }
    },
    generateStream: roomId => {
      if (timestamp === undefined) {
        timestamp = new Date().getTime();
      }
      const name = uuidv1();

      const path = config.folder.audioRaw + '_' + roomId + '/' + name + '.wav';
      const streamData = {
        wstream: fs.createWriteStream(path),
        pathFile: path,
        fileName: name,
        room: roomId,
        timestamp: new Date().getTime()
      };
      if (userConf[roomId] === undefined) {
        console.log('INFO ROOM ADD USER', roomId);
        userConf[roomId] = 0;
      }
      userConf[roomId]++;
      return streamData;
    },

    endStream: (streamData, roomId) => {
      const toSecond = 1000;
      streamData.wstream.close();
      const delay = ((timestamp - streamData.timestamp) / toSecond) * -1;
      try {
        console.log('generate silence');
        ffmpeg()
          .input('anullsrc=sample_rate=' + config.audio.srate)
          .inputFormat('lavfi')
          .duration(delay)
          .output(config.folder.audioWav + '_' + streamData.room + '/silence_' + streamData.fileName + '.wav')
          .on('end', () => {
            console.log('generate audio file');
            ffmpeg(streamData.pathFile)
              .toFormat(config.audio.format)
              .noVideo()
              .audioBitrate(config.audio.bitrate)
              .audioChannels(config.audio.channels)
              .audioFrequency(config.audio.srate)
              .save(config.folder.audioWav + '_' + streamData.room + '/' + streamData.fileName + '.wav')
              .on('error', err => {
                --userConf[streamData.room];
                console.log('An error occurred: ' + err.message);
              })
              .on('end', () => {
                console.log('merge audio and silence');
                ffmpeg()
                  .input(config.folder.audioWav + '_' + streamData.room + '/silence_' + streamData.fileName + '.wav')
                  .input(config.folder.audioWav + '_' + streamData.room + '/' + streamData.fileName + '.wav')

                  .on('error', err => {
                    console.log('An error occurred: ' + err.message);
                  })
                  .on('end', () => {
                    console.log('Merging finished !' + streamData.fileName);
                    if (--userConf[streamData.room] === 0 && resumeConfig.isUse === true) {
                      console.log('generate room audio');
                      resumeGenerator(streamData.room);
                    }
                  })
                  .mergeToFile(config.folder.audioMerged + '_' + streamData.room + '/' + streamData.fileName + '.wav', config.folder.audio + '_' + streamData.room);
              });
          })
          .run();
      } catch (err) {
        // Sometimes, the client will close the connection unexpectedly
        // (e.g. when stopping). This is expected
      }
    }
  };
};
